@@ -116,8 +116,6 @@ class Imported_KeyStore(Software_KeyStore):
     def __init__(self, d):
         Software_KeyStore.__init__(self)
         self.keypairs = d.get('keypairs', {})
-        self.receiving_pubkeys = self.keypairs.keys()
-        self.change_pubkeys = []
 
     def is_deterministic(self):
         return False
@@ -138,35 +136,22 @@ class Imported_KeyStore(Software_KeyStore):
         return True
 
     def check_password(self, password):
-        self.get_private_key((0,0), password)
+        pubkey = self.keypairs.keys()[0]
+        self.get_private_key(pubkey, password)
 
     def import_key(self, sec, password):
-        if not self.can_import():
-            raise BaseException('This wallet cannot import private keys')
         try:
             pubkey = public_key_from_private_key(sec)
         except Exception:
             raise BaseException('Invalid private key')
-        if pubkey in self.keypairs:
-            raise BaseException('Private key already in keystore')
-        self.keypairs[pubkey] = sec
+        # allow overwrite
+        self.keypairs[pubkey] = pw_encode(sec, password)
         return pubkey
 
     def delete_imported_key(self, key):
         self.keypairs.pop(key)
 
-    def get_public_key(self, sequence):
-        for_change, i = sequence
-        pubkey = (self.change_pubkeys if for_change else self.receiving_pubkeys)[i]
-        return pubkey
-
-    def get_xpubkey(self, c, i):
-        return self.get_public_key((c,i))
-
-    def get_private_key(self, sequence, password):
-        for_change, i = sequence
-        assert for_change == 0
-        pubkey = self.receiving_pubkeys[i]
+    def get_private_key(self, pubkey, password):
         pk = pw_decode(self.keypairs[pubkey], password)
         # this checks the password
         if pubkey != public_key_from_private_key(pk):
@@ -175,15 +160,14 @@ class Imported_KeyStore(Software_KeyStore):
 
     def get_pubkey_derivation(self, x_pubkey):
         if x_pubkey[0:2] in ['02', '03', '04']:
-            if x_pubkey in self.receiving_pubkeys:
-                i = self.receiving_pubkeys.index(x_pubkey)
-                return (False, i)
+            if x_pubkey in self.keypairs.keys():
+                return x_pubkey
         elif x_pubkey[0:2] == 'fd':
             # fixme: this assumes p2pkh
             _, addr = xpubkey_to_address(x_pubkey)
-            for i, pubkey in enumerate(self.receiving_pubkeys):
+            for pubkey in self.keypairs.keys():
                 if public_key_to_bc_address(pubkey.decode('hex')) == addr:
-                    return (False, i)
+                    return pubkey
 
     def update_password(self, old_password, new_password):
         self.check_password(old_password)
@@ -216,6 +200,9 @@ class Deterministic_KeyStore(Software_KeyStore):
     def has_seed(self):
         return bool(self.seed)
 
+    def is_watching_only(self):
+        return not self.has_seed()
+
     def can_change_password(self):
         return not self.is_watching_only()
 
@@ -225,10 +212,10 @@ class Deterministic_KeyStore(Software_KeyStore):
         self.seed = self.format_seed(seed)
 
     def get_seed(self, password):
-        return pw_decode(self.seed, password).encode('utf8')
+        return pw_decode(self.seed, password)
 
     def get_passphrase(self, password):
-        return pw_decode(self.passphrase, password).encode('utf8') if self.passphrase else ''
+        return pw_decode(self.passphrase, password) if self.passphrase else ''
 
 
 
@@ -357,6 +344,9 @@ class Old_KeyStore(Deterministic_KeyStore):
         Deterministic_KeyStore.__init__(self, d)
         self.mpk = d.get('mpk')
 
+    def get_seed(self, password):
+        return pw_decode(self.seed, password).encode('utf8')
+
     def dump(self):
         d = Deterministic_KeyStore.dump(self)
         d['mpk'] = self.mpk
